package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberCard;
import com.czsj.account.service.ICzsjMemberCardService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员证件信息Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/card")
public class CzsjMemberCardController extends BaseController
{
    @Autowired
    private ICzsjMemberCardService czsjMemberCardService;

    /**
     * 查询会员证件信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:card:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberCard czsjMemberCard)
    {
        startPage();
        List<CzsjMemberCard> list = czsjMemberCardService.selectCzsjMemberCardList(czsjMemberCard);
        return getDataTable(list);
    }

    /**
     * 导出会员证件信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:card:export')")
    @Log(title = "会员证件信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberCard czsjMemberCard)
    {
        List<CzsjMemberCard> list = czsjMemberCardService.selectCzsjMemberCardList(czsjMemberCard);
        ExcelUtil<CzsjMemberCard> util = new ExcelUtil<CzsjMemberCard>(CzsjMemberCard.class);
        util.exportExcel(response, list, "会员证件信息数据");
    }

    /**
     * 获取会员证件信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:card:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Integer uid)
    {
        return success(czsjMemberCardService.selectCzsjMemberCardByUid(uid));
    }

    /**
     * 新增会员证件信息
     */
    @PreAuthorize("@ss.hasPermi('account:card:add')")
    @Log(title = "会员证件信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberCard czsjMemberCard)
    {
        return toAjax(czsjMemberCardService.insertCzsjMemberCard(czsjMemberCard));
    }

    /**
     * 修改会员证件信息
     */
    @PreAuthorize("@ss.hasPermi('account:card:edit')")
    @Log(title = "会员证件信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberCard czsjMemberCard)
    {
        return toAjax(czsjMemberCardService.updateCzsjMemberCard(czsjMemberCard));
    }

    /**
     * 删除会员证件信息
     */
    @PreAuthorize("@ss.hasPermi('account:card:remove')")
    @Log(title = "会员证件信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Integer[] uids)
    {
        return toAjax(czsjMemberCardService.deleteCzsjMemberCardByUids(uids));
    }
}
