package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMember;
import com.czsj.account.service.ICzsjMemberService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员信息Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/member")
public class CzsjMemberController extends BaseController
{
    @Autowired
    private ICzsjMemberService czsjMemberService;

    /**
     * 查询会员信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:member:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMember czsjMember)
    {
        startPage();
        List<CzsjMember> list = czsjMemberService.selectCzsjMemberList(czsjMember);
        return getDataTable(list);
    }

    /**
     * 导出会员信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:member:export')")
    @Log(title = "会员信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMember czsjMember)
    {
        List<CzsjMember> list = czsjMemberService.selectCzsjMemberList(czsjMember);
        ExcelUtil<CzsjMember> util = new ExcelUtil<CzsjMember>(CzsjMember.class);
        util.exportExcel(response, list, "会员信息数据");
    }

    /**
     * 获取会员信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:member:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberService.selectCzsjMemberByUid(uid));
    }

    /**
     * 新增会员信息
     */
    @PreAuthorize("@ss.hasPermi('account:member:add')")
    @Log(title = "会员信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMember czsjMember)
    {
        return toAjax(czsjMemberService.insertCzsjMember(czsjMember));
    }

    /**
     * 修改会员信息
     */
    @PreAuthorize("@ss.hasPermi('account:member:edit')")
    @Log(title = "会员信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMember czsjMember)
    {
        return toAjax(czsjMemberService.updateCzsjMember(czsjMember));
    }

    /**
     * 删除会员信息
     */
    @PreAuthorize("@ss.hasPermi('account:member:remove')")
    @Log(title = "会员信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberService.deleteCzsjMemberByUids(uids));
    }
}
