package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberLocus;
import com.czsj.account.service.ICzsjMemberLocusService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员轨迹信息Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/locus")
public class CzsjMemberLocusController extends BaseController
{
    @Autowired
    private ICzsjMemberLocusService czsjMemberLocusService;

    /**
     * 查询会员轨迹信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:locus:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberLocus czsjMemberLocus)
    {
        startPage();
        List<CzsjMemberLocus> list = czsjMemberLocusService.selectCzsjMemberLocusList(czsjMemberLocus);
        return getDataTable(list);
    }

    /**
     * 导出会员轨迹信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:locus:export')")
    @Log(title = "会员轨迹信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberLocus czsjMemberLocus)
    {
        List<CzsjMemberLocus> list = czsjMemberLocusService.selectCzsjMemberLocusList(czsjMemberLocus);
        ExcelUtil<CzsjMemberLocus> util = new ExcelUtil<CzsjMemberLocus>(CzsjMemberLocus.class);
        util.exportExcel(response, list, "会员轨迹信息数据");
    }

    /**
     * 获取会员轨迹信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:locus:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberLocusService.selectCzsjMemberLocusByUid(uid));
    }

    /**
     * 新增会员轨迹信息
     */
    @PreAuthorize("@ss.hasPermi('account:locus:add')")
    @Log(title = "会员轨迹信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberLocus czsjMemberLocus)
    {
        return toAjax(czsjMemberLocusService.insertCzsjMemberLocus(czsjMemberLocus));
    }

    /**
     * 修改会员轨迹信息
     */
    @PreAuthorize("@ss.hasPermi('account:locus:edit')")
    @Log(title = "会员轨迹信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberLocus czsjMemberLocus)
    {
        return toAjax(czsjMemberLocusService.updateCzsjMemberLocus(czsjMemberLocus));
    }

    /**
     * 删除会员轨迹信息
     */
    @PreAuthorize("@ss.hasPermi('account:locus:remove')")
    @Log(title = "会员轨迹信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberLocusService.deleteCzsjMemberLocusByUids(uids));
    }
}
