package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberRelation;
import com.czsj.account.service.ICzsjMemberRelationService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员关系Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/relation")
public class CzsjMemberRelationController extends BaseController
{
    @Autowired
    private ICzsjMemberRelationService czsjMemberRelationService;

    /**
     * 查询会员关系列表
     */
    @PreAuthorize("@ss.hasPermi('account:relation:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberRelation czsjMemberRelation)
    {
        startPage();
        List<CzsjMemberRelation> list = czsjMemberRelationService.selectCzsjMemberRelationList(czsjMemberRelation);
        return getDataTable(list);
    }

    /**
     * 导出会员关系列表
     */
    @PreAuthorize("@ss.hasPermi('account:relation:export')")
    @Log(title = "会员关系", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberRelation czsjMemberRelation)
    {
        List<CzsjMemberRelation> list = czsjMemberRelationService.selectCzsjMemberRelationList(czsjMemberRelation);
        ExcelUtil<CzsjMemberRelation> util = new ExcelUtil<CzsjMemberRelation>(CzsjMemberRelation.class);
        util.exportExcel(response, list, "会员关系数据");
    }

    /**
     * 获取会员关系详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:relation:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberRelationService.selectCzsjMemberRelationByUid(uid));
    }

    /**
     * 新增会员关系
     */
    @PreAuthorize("@ss.hasPermi('account:relation:add')")
    @Log(title = "会员关系", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberRelation czsjMemberRelation)
    {
        return toAjax(czsjMemberRelationService.insertCzsjMemberRelation(czsjMemberRelation));
    }

    /**
     * 修改会员关系
     */
    @PreAuthorize("@ss.hasPermi('account:relation:edit')")
    @Log(title = "会员关系", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberRelation czsjMemberRelation)
    {
        return toAjax(czsjMemberRelationService.updateCzsjMemberRelation(czsjMemberRelation));
    }

    /**
     * 删除会员关系
     */
    @PreAuthorize("@ss.hasPermi('account:relation:remove')")
    @Log(title = "会员关系", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberRelationService.deleteCzsjMemberRelationByUids(uids));
    }
}
