package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberSign;
import com.czsj.account.service.ICzsjMemberSignService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员签到记录Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/sign")
public class CzsjMemberSignController extends BaseController
{
    @Autowired
    private ICzsjMemberSignService czsjMemberSignService;

    /**
     * 查询会员签到记录列表
     */
    @PreAuthorize("@ss.hasPermi('account:sign:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberSign czsjMemberSign)
    {
        startPage();
        List<CzsjMemberSign> list = czsjMemberSignService.selectCzsjMemberSignList(czsjMemberSign);
        return getDataTable(list);
    }

    /**
     * 导出会员签到记录列表
     */
    @PreAuthorize("@ss.hasPermi('account:sign:export')")
    @Log(title = "会员签到记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberSign czsjMemberSign)
    {
        List<CzsjMemberSign> list = czsjMemberSignService.selectCzsjMemberSignList(czsjMemberSign);
        ExcelUtil<CzsjMemberSign> util = new ExcelUtil<CzsjMemberSign>(CzsjMemberSign.class);
        util.exportExcel(response, list, "会员签到记录数据");
    }

    /**
     * 获取会员签到记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:sign:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberSignService.selectCzsjMemberSignByUid(uid));
    }

    /**
     * 新增会员签到记录
     */
    @PreAuthorize("@ss.hasPermi('account:sign:add')")
    @Log(title = "会员签到记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberSign czsjMemberSign)
    {
        return toAjax(czsjMemberSignService.insertCzsjMemberSign(czsjMemberSign));
    }

    /**
     * 修改会员签到记录
     */
    @PreAuthorize("@ss.hasPermi('account:sign:edit')")
    @Log(title = "会员签到记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberSign czsjMemberSign)
    {
        return toAjax(czsjMemberSignService.updateCzsjMemberSign(czsjMemberSign));
    }

    /**
     * 删除会员签到记录
     */
    @PreAuthorize("@ss.hasPermi('account:sign:remove')")
    @Log(title = "会员签到记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberSignService.deleteCzsjMemberSignByUids(uids));
    }
}
