package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberTagGroup;
import com.czsj.account.service.ICzsjMemberTagGroupService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员标签组Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/group")
public class CzsjMemberTagGroupController extends BaseController
{
    @Autowired
    private ICzsjMemberTagGroupService czsjMemberTagGroupService;

    /**
     * 查询会员标签组列表
     */
    @PreAuthorize("@ss.hasPermi('account:group:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberTagGroup czsjMemberTagGroup)
    {
        startPage();
        List<CzsjMemberTagGroup> list = czsjMemberTagGroupService.selectCzsjMemberTagGroupList(czsjMemberTagGroup);
        return getDataTable(list);
    }

    /**
     * 导出会员标签组列表
     */
    @PreAuthorize("@ss.hasPermi('account:group:export')")
    @Log(title = "会员标签组", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberTagGroup czsjMemberTagGroup)
    {
        List<CzsjMemberTagGroup> list = czsjMemberTagGroupService.selectCzsjMemberTagGroupList(czsjMemberTagGroup);
        ExcelUtil<CzsjMemberTagGroup> util = new ExcelUtil<CzsjMemberTagGroup>(CzsjMemberTagGroup.class);
        util.exportExcel(response, list, "会员标签组数据");
    }

    /**
     * 获取会员标签组详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:group:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberTagGroupService.selectCzsjMemberTagGroupByUid(uid));
    }

    /**
     * 新增会员标签组
     */
    @PreAuthorize("@ss.hasPermi('account:group:add')")
    @Log(title = "会员标签组", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberTagGroup czsjMemberTagGroup)
    {
        return toAjax(czsjMemberTagGroupService.insertCzsjMemberTagGroup(czsjMemberTagGroup));
    }

    /**
     * 修改会员标签组
     */
    @PreAuthorize("@ss.hasPermi('account:group:edit')")
    @Log(title = "会员标签组", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberTagGroup czsjMemberTagGroup)
    {
        return toAjax(czsjMemberTagGroupService.updateCzsjMemberTagGroup(czsjMemberTagGroup));
    }

    /**
     * 删除会员标签组
     */
    @PreAuthorize("@ss.hasPermi('account:group:remove')")
    @Log(title = "会员标签组", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberTagGroupService.deleteCzsjMemberTagGroupByUids(uids));
    }
}
