package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberWxFans;
import com.czsj.account.service.ICzsjMemberWxFansService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员微信粉丝Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/fans")
public class CzsjMemberWxFansController extends BaseController
{
    @Autowired
    private ICzsjMemberWxFansService czsjMemberWxFansService;

    /**
     * 查询会员微信粉丝列表
     */
    @PreAuthorize("@ss.hasPermi('account:fans:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberWxFans czsjMemberWxFans)
    {
        startPage();
        List<CzsjMemberWxFans> list = czsjMemberWxFansService.selectCzsjMemberWxFansList(czsjMemberWxFans);
        return getDataTable(list);
    }

    /**
     * 导出会员微信粉丝列表
     */
    @PreAuthorize("@ss.hasPermi('account:fans:export')")
    @Log(title = "会员微信粉丝", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberWxFans czsjMemberWxFans)
    {
        List<CzsjMemberWxFans> list = czsjMemberWxFansService.selectCzsjMemberWxFansList(czsjMemberWxFans);
        ExcelUtil<CzsjMemberWxFans> util = new ExcelUtil<CzsjMemberWxFans>(CzsjMemberWxFans.class);
        util.exportExcel(response, list, "会员微信粉丝数据");
    }

    /**
     * 获取会员微信粉丝详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:fans:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberWxFansService.selectCzsjMemberWxFansByUid(uid));
    }

    /**
     * 新增会员微信粉丝
     */
    @PreAuthorize("@ss.hasPermi('account:fans:add')")
    @Log(title = "会员微信粉丝", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberWxFans czsjMemberWxFans)
    {
        return toAjax(czsjMemberWxFansService.insertCzsjMemberWxFans(czsjMemberWxFans));
    }

    /**
     * 修改会员微信粉丝
     */
    @PreAuthorize("@ss.hasPermi('account:fans:edit')")
    @Log(title = "会员微信粉丝", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberWxFans czsjMemberWxFans)
    {
        return toAjax(czsjMemberWxFansService.updateCzsjMemberWxFans(czsjMemberWxFans));
    }

    /**
     * 删除会员微信粉丝
     */
    @PreAuthorize("@ss.hasPermi('account:fans:remove')")
    @Log(title = "会员微信粉丝", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberWxFansService.deleteCzsjMemberWxFansByUids(uids));
    }
}
