package com.czsj.web.controller.account;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.czsj.common.annotation.Log;
import com.czsj.common.core.controller.BaseController;
import com.czsj.common.core.domain.AjaxResult;
import com.czsj.common.enums.BusinessType;
import com.czsj.account.domain.CzsjMemberWxSource;
import com.czsj.account.service.ICzsjMemberWxSourceService;
import com.czsj.common.utils.poi.ExcelUtil;
import com.czsj.common.core.page.TableDataInfo;

/**
 * 会员微信来源信息Controller
 * 
 * @author czsj
 * @date 2024-12-01
 */
@RestController
@RequestMapping("/account/source")
public class CzsjMemberWxSourceController extends BaseController
{
    @Autowired
    private ICzsjMemberWxSourceService czsjMemberWxSourceService;

    /**
     * 查询会员微信来源信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:source:list')")
    @GetMapping("/list")
    public TableDataInfo list(CzsjMemberWxSource czsjMemberWxSource)
    {
        startPage();
        List<CzsjMemberWxSource> list = czsjMemberWxSourceService.selectCzsjMemberWxSourceList(czsjMemberWxSource);
        return getDataTable(list);
    }

    /**
     * 导出会员微信来源信息列表
     */
    @PreAuthorize("@ss.hasPermi('account:source:export')")
    @Log(title = "会员微信来源信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, CzsjMemberWxSource czsjMemberWxSource)
    {
        List<CzsjMemberWxSource> list = czsjMemberWxSourceService.selectCzsjMemberWxSourceList(czsjMemberWxSource);
        ExcelUtil<CzsjMemberWxSource> util = new ExcelUtil<CzsjMemberWxSource>(CzsjMemberWxSource.class);
        util.exportExcel(response, list, "会员微信来源信息数据");
    }

    /**
     * 获取会员微信来源信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('account:source:query')")
    @GetMapping(value = "/{uid}")
    public AjaxResult getInfo(@PathVariable("uid") Long uid)
    {
        return success(czsjMemberWxSourceService.selectCzsjMemberWxSourceByUid(uid));
    }

    /**
     * 新增会员微信来源信息
     */
    @PreAuthorize("@ss.hasPermi('account:source:add')")
    @Log(title = "会员微信来源信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody CzsjMemberWxSource czsjMemberWxSource)
    {
        return toAjax(czsjMemberWxSourceService.insertCzsjMemberWxSource(czsjMemberWxSource));
    }

    /**
     * 修改会员微信来源信息
     */
    @PreAuthorize("@ss.hasPermi('account:source:edit')")
    @Log(title = "会员微信来源信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody CzsjMemberWxSource czsjMemberWxSource)
    {
        return toAjax(czsjMemberWxSourceService.updateCzsjMemberWxSource(czsjMemberWxSource));
    }

    /**
     * 删除会员微信来源信息
     */
    @PreAuthorize("@ss.hasPermi('account:source:remove')")
    @Log(title = "会员微信来源信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{uids}")
    public AjaxResult remove(@PathVariable Long[] uids)
    {
        return toAjax(czsjMemberWxSourceService.deleteCzsjMemberWxSourceByUids(uids));
    }
}
