package com.czsj.account.service.impl;

import java.util.List;
import com.czsj.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.czsj.account.mapper.CzsjMemberCardMapper;
import com.czsj.account.domain.CzsjMemberCard;
import com.czsj.account.service.ICzsjMemberCardService;

/**
 * 会员证件信息Service业务层处理
 * 
 * @author czsj
 * @date 2024-12-01
 */
@Service
public class CzsjMemberCardServiceImpl implements ICzsjMemberCardService 
{
    @Autowired
    private CzsjMemberCardMapper czsjMemberCardMapper;

    /**
     * 查询会员证件信息
     * 
     * @param uid 会员证件信息主键
     * @return 会员证件信息
     */
    @Override
    public CzsjMemberCard selectCzsjMemberCardByUid(Integer uid)
    {
        return czsjMemberCardMapper.selectCzsjMemberCardByUid(uid);
    }

    /**
     * 查询会员证件信息列表
     * 
     * @param czsjMemberCard 会员证件信息
     * @return 会员证件信息
     */
    @Override
    public List<CzsjMemberCard> selectCzsjMemberCardList(CzsjMemberCard czsjMemberCard)
    {
        return czsjMemberCardMapper.selectCzsjMemberCardList(czsjMemberCard);
    }

    /**
     * 新增会员证件信息
     * 
     * @param czsjMemberCard 会员证件信息
     * @return 结果
     */
    @Override
    public int insertCzsjMemberCard(CzsjMemberCard czsjMemberCard)
    {
        czsjMemberCard.setCreateTime(DateUtils.getNowDate());
        return czsjMemberCardMapper.insertCzsjMemberCard(czsjMemberCard);
    }

    /**
     * 修改会员证件信息
     * 
     * @param czsjMemberCard 会员证件信息
     * @return 结果
     */
    @Override
    public int updateCzsjMemberCard(CzsjMemberCard czsjMemberCard)
    {
        czsjMemberCard.setUpdateTime(DateUtils.getNowDate());
        return czsjMemberCardMapper.updateCzsjMemberCard(czsjMemberCard);
    }

    /**
     * 批量删除会员证件信息
     * 
     * @param uids 需要删除的会员证件信息主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberCardByUids(Integer[] uids)
    {
        return czsjMemberCardMapper.deleteCzsjMemberCardByUids(uids);
    }

    /**
     * 删除会员证件信息信息
     * 
     * @param uid 会员证件信息主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberCardByUid(Integer uid)
    {
        return czsjMemberCardMapper.deleteCzsjMemberCardByUid(uid);
    }
}
