package com.czsj.account.service.impl;

import java.util.List;
import com.czsj.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.czsj.account.mapper.CzsjMemberLocusMapper;
import com.czsj.account.domain.CzsjMemberLocus;
import com.czsj.account.service.ICzsjMemberLocusService;

/**
 * 会员轨迹信息Service业务层处理
 * 
 * @author czsj
 * @date 2024-12-01
 */
@Service
public class CzsjMemberLocusServiceImpl implements ICzsjMemberLocusService 
{
    @Autowired
    private CzsjMemberLocusMapper czsjMemberLocusMapper;

    /**
     * 查询会员轨迹信息
     * 
     * @param uid 会员轨迹信息主键
     * @return 会员轨迹信息
     */
    @Override
    public CzsjMemberLocus selectCzsjMemberLocusByUid(Long uid)
    {
        return czsjMemberLocusMapper.selectCzsjMemberLocusByUid(uid);
    }

    /**
     * 查询会员轨迹信息列表
     * 
     * @param czsjMemberLocus 会员轨迹信息
     * @return 会员轨迹信息
     */
    @Override
    public List<CzsjMemberLocus> selectCzsjMemberLocusList(CzsjMemberLocus czsjMemberLocus)
    {
        return czsjMemberLocusMapper.selectCzsjMemberLocusList(czsjMemberLocus);
    }

    /**
     * 新增会员轨迹信息
     * 
     * @param czsjMemberLocus 会员轨迹信息
     * @return 结果
     */
    @Override
    public int insertCzsjMemberLocus(CzsjMemberLocus czsjMemberLocus)
    {
        czsjMemberLocus.setCreateTime(DateUtils.getNowDate());
        return czsjMemberLocusMapper.insertCzsjMemberLocus(czsjMemberLocus);
    }

    /**
     * 修改会员轨迹信息
     * 
     * @param czsjMemberLocus 会员轨迹信息
     * @return 结果
     */
    @Override
    public int updateCzsjMemberLocus(CzsjMemberLocus czsjMemberLocus)
    {
        czsjMemberLocus.setUpdateTime(DateUtils.getNowDate());
        return czsjMemberLocusMapper.updateCzsjMemberLocus(czsjMemberLocus);
    }

    /**
     * 批量删除会员轨迹信息
     * 
     * @param uids 需要删除的会员轨迹信息主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberLocusByUids(Long[] uids)
    {
        return czsjMemberLocusMapper.deleteCzsjMemberLocusByUids(uids);
    }

    /**
     * 删除会员轨迹信息信息
     * 
     * @param uid 会员轨迹信息主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberLocusByUid(Long uid)
    {
        return czsjMemberLocusMapper.deleteCzsjMemberLocusByUid(uid);
    }
}
