package com.czsj.account.service.impl;

import java.util.List;
import com.czsj.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.czsj.account.mapper.CzsjMemberTagGroupMapper;
import com.czsj.account.domain.CzsjMemberTagGroup;
import com.czsj.account.service.ICzsjMemberTagGroupService;

/**
 * 会员标签组Service业务层处理
 * 
 * @author czsj
 * @date 2024-12-01
 */
@Service
public class CzsjMemberTagGroupServiceImpl implements ICzsjMemberTagGroupService 
{
    @Autowired
    private CzsjMemberTagGroupMapper czsjMemberTagGroupMapper;

    /**
     * 查询会员标签组
     * 
     * @param uid 会员标签组主键
     * @return 会员标签组
     */
    @Override
    public CzsjMemberTagGroup selectCzsjMemberTagGroupByUid(Long uid)
    {
        return czsjMemberTagGroupMapper.selectCzsjMemberTagGroupByUid(uid);
    }

    /**
     * 查询会员标签组列表
     * 
     * @param czsjMemberTagGroup 会员标签组
     * @return 会员标签组
     */
    @Override
    public List<CzsjMemberTagGroup> selectCzsjMemberTagGroupList(CzsjMemberTagGroup czsjMemberTagGroup)
    {
        return czsjMemberTagGroupMapper.selectCzsjMemberTagGroupList(czsjMemberTagGroup);
    }

    /**
     * 新增会员标签组
     * 
     * @param czsjMemberTagGroup 会员标签组
     * @return 结果
     */
    @Override
    public int insertCzsjMemberTagGroup(CzsjMemberTagGroup czsjMemberTagGroup)
    {
        czsjMemberTagGroup.setCreateTime(DateUtils.getNowDate());
        return czsjMemberTagGroupMapper.insertCzsjMemberTagGroup(czsjMemberTagGroup);
    }

    /**
     * 修改会员标签组
     * 
     * @param czsjMemberTagGroup 会员标签组
     * @return 结果
     */
    @Override
    public int updateCzsjMemberTagGroup(CzsjMemberTagGroup czsjMemberTagGroup)
    {
        czsjMemberTagGroup.setUpdateTime(DateUtils.getNowDate());
        return czsjMemberTagGroupMapper.updateCzsjMemberTagGroup(czsjMemberTagGroup);
    }

    /**
     * 批量删除会员标签组
     * 
     * @param uids 需要删除的会员标签组主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberTagGroupByUids(Long[] uids)
    {
        return czsjMemberTagGroupMapper.deleteCzsjMemberTagGroupByUids(uids);
    }

    /**
     * 删除会员标签组信息
     * 
     * @param uid 会员标签组主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberTagGroupByUid(Long uid)
    {
        return czsjMemberTagGroupMapper.deleteCzsjMemberTagGroupByUid(uid);
    }
}
