package com.czsj.account.service.impl;

import java.util.List;
import com.czsj.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.czsj.account.mapper.CzsjMemberWxSourceMapper;
import com.czsj.account.domain.CzsjMemberWxSource;
import com.czsj.account.service.ICzsjMemberWxSourceService;

/**
 * 会员微信来源信息Service业务层处理
 * 
 * @author czsj
 * @date 2024-12-01
 */
@Service
public class CzsjMemberWxSourceServiceImpl implements ICzsjMemberWxSourceService 
{
    @Autowired
    private CzsjMemberWxSourceMapper czsjMemberWxSourceMapper;

    /**
     * 查询会员微信来源信息
     * 
     * @param uid 会员微信来源信息主键
     * @return 会员微信来源信息
     */
    @Override
    public CzsjMemberWxSource selectCzsjMemberWxSourceByUid(Long uid)
    {
        return czsjMemberWxSourceMapper.selectCzsjMemberWxSourceByUid(uid);
    }

    /**
     * 查询会员微信来源信息列表
     * 
     * @param czsjMemberWxSource 会员微信来源信息
     * @return 会员微信来源信息
     */
    @Override
    public List<CzsjMemberWxSource> selectCzsjMemberWxSourceList(CzsjMemberWxSource czsjMemberWxSource)
    {
        return czsjMemberWxSourceMapper.selectCzsjMemberWxSourceList(czsjMemberWxSource);
    }

    /**
     * 新增会员微信来源信息
     * 
     * @param czsjMemberWxSource 会员微信来源信息
     * @return 结果
     */
    @Override
    public int insertCzsjMemberWxSource(CzsjMemberWxSource czsjMemberWxSource)
    {
        czsjMemberWxSource.setCreateTime(DateUtils.getNowDate());
        return czsjMemberWxSourceMapper.insertCzsjMemberWxSource(czsjMemberWxSource);
    }

    /**
     * 修改会员微信来源信息
     * 
     * @param czsjMemberWxSource 会员微信来源信息
     * @return 结果
     */
    @Override
    public int updateCzsjMemberWxSource(CzsjMemberWxSource czsjMemberWxSource)
    {
        czsjMemberWxSource.setUpdateTime(DateUtils.getNowDate());
        return czsjMemberWxSourceMapper.updateCzsjMemberWxSource(czsjMemberWxSource);
    }

    /**
     * 批量删除会员微信来源信息
     * 
     * @param uids 需要删除的会员微信来源信息主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberWxSourceByUids(Long[] uids)
    {
        return czsjMemberWxSourceMapper.deleteCzsjMemberWxSourceByUids(uids);
    }

    /**
     * 删除会员微信来源信息信息
     * 
     * @param uid 会员微信来源信息主键
     * @return 结果
     */
    @Override
    public int deleteCzsjMemberWxSourceByUid(Long uid)
    {
        return czsjMemberWxSourceMapper.deleteCzsjMemberWxSourceByUid(uid);
    }
}
